@@ -1,8 +1,10 @@
 from abc import ABC, abstractmethod
 from collections import deque
 from datetime import timedelta
+import enum
 from eventkit import Event
 import math
+import pandas as pd
 from typing import List, Deque, Dict
 
 from . import brokerinterface
@@ -10,6 +12,13 @@ from . import datacontainer
 from . import utils
 
 
+class WeightingScheme(enum.Enum):
+    """
+    Enum class for portfolio weighting scheme
+    """
+    Equal = 1
+
+
 class TradeCombos:
     """
     Class for trade combos, which is a basket of contracts traded by a strategy.
@@ -77,7 +86,8 @@ class StrategyBase(ABC):
                 trade_combo: TradeCombos,
                 warmup_length: int,
                 initial_capital: float = 100.0,
-                order_manager: brokerinterface.OrderManager = None):
+                order_manager: brokerinterface.OrderManagerBase = None,
+                **kwargs):
         """
         Create derived strategy object.
 
@@ -91,7 +101,7 @@ class StrategyBase(ABC):
             TradeCombos object that defines the combos to be traded. Note that if combo definition is missing,
             all trades from this strategy will be under a generic combo.
         warmup_length: int
-            Number of data points to 'burn'
+            Number of data points to 'burn'.
         initial_capital: float
             Initial capital.
         order_manager: OrderManager
@@ -143,7 +153,7 @@ class StrategyBase(ABC):
                  trade_combo: TradeCombos,
                  warmup_length: int,
                  initial_capital: float = 100.0,
-                 order_manager: brokerinterface.OrderManager = None) -> None:
+                 order_manager: brokerinterface.OrderManagerBase = None) -> None:
         pass
 
     def _initialize_signal_time_storage(self, input_signal_array: List[Event]) -> None:
@@ -263,10 +273,39 @@ class StrategyBase(ABC):
         self.get_contract_time_by_name(new_data.get_name()).append(new_data.get_time())
         if len(self.get_contract_time_by_name(new_data.get_name())) > 1:
             self.get_contract_time_by_name(new_data.get_name()).popleft()
+        # Update PM combo mtm price
+        if self._is_live_trading:
+            # Build a dataframe
+            combo_mtm = pd.DataFrame(columns=['strategy_name', 'combo_name', 'combo_mtm_price'])
+            for combo_name in self._combo_def.keys():
+                combo_mtm = combo_mtm.append({'strategy_name': self._strategy_name,
+                                              'combo_name': combo_name,
+                                              'combo_mtm_price': self.get_combo_mtm_price(combo_name)},
+                                             ignore_index=True)
+            # Update
+            self._order_manager.get_portfolio_manager().update_combo_mtm_price(combo_mtm)
+        pass
 
     def get_mtm_history(self) -> List[float]:
         return self._mtm_history
 
+    def get_mtm_price_by_name(self, data_name: str) -> float:
+        """Return the MTM price of a contract."""
+        return self._mtm_price[data_name]
+
+    def get_combo_mtm_price(self, combo_name: str) -> float:
+        """
+        Retrieve combo mtm price.
+
+
+        Parameters
+        ----------
+        combo_name: str
+            Combo name.
+        """
+        combo_def = self._combo_def[combo_name]
+        return sum(i[0] * i[1] for i in zip(combo_def, self._mtm_price.values()))
+
     def get_contract_time_by_name(self, data_name: str) -> Deque:
         return self._contract_time_storage[data_name]
 
@@ -305,20 +344,35 @@ class StrategyBase(ABC):
         Send pending orders and use latest price to calculate average cost.
         """
         for combo_name, amount in self._combo_order.items():
-            combo_def = self._combo_def[combo_name]
-            self._combo_positions[combo_name] += amount
-            for weight_i, contract_i in zip(combo_def, self._contract_array):
+            if combo_name in self._combo_def:
+                combo_def = self._combo_def[combo_name]
+                self._combo_positions[combo_name] += amount
+                for weight_i, contract_i in zip(combo_def, self._contract_array):
+                    # Adjust cash account
+                    name_i = contract_i.name()
+                    self._contract_positions[self.CASH_ACCOUNT_NAME] -= amount * weight_i * self._mtm_price[name_i]
+                    # Calculate average cost
+                    self._average_entry_price[name_i] = self._calculate_new_average_entry_price(
+                        self._average_entry_price[name_i],
+                        self._mtm_price[name_i],
+                        self._contract_positions[name_i],
+                        amount * weight_i)
+                    # Modify current position
+                    self._contract_positions[name_i] += amount * weight_i
+            else:
+                # Some strategies do not have predefined combos;
+                # for them we simply place orders for individual contract.
+                contract_name = combo_name
                 # Adjust cash account
-                name_i = contract_i.name()
-                self._contract_positions[self.CASH_ACCOUNT_NAME] -= amount * weight_i * self._mtm_price[name_i]
+                self._contract_positions[self.CASH_ACCOUNT_NAME] -= amount * self._mtm_price[contract_name]
                 # Calculate average cost
-                self._average_entry_price[name_i] = self._calculate_new_average_entry_price(
-                    self._average_entry_price[name_i],
-                    self._mtm_price[name_i],
-                    self._contract_positions[name_i],
-                    amount * weight_i)
+                self._average_entry_price[contract_name] = self._calculate_new_average_entry_price(
+                    self._average_entry_price[contract_name],
+                    self._mtm_price[contract_name],
+                    self._contract_positions[contract_name],
+                    amount)
                 # Modify current position
-                self._contract_positions[name_i] += amount * weight_i
+                self._contract_positions[contract_name] += amount
             # Reset pending order
             self._combo_order[combo_name] = 0.0
 
@@ -326,14 +380,26 @@ class StrategyBase(ABC):
         """
         Send pending orders to broker.
         """
-        pass
-
-    #         self._order_manager.place_order(self._strategy_name,
-    #                                        self._contract_array,
-    #                                        )
-    #                     weight_array: List[float],
-    #                     combo_unit: float,
-    #                     combo_name: str
+        contract_array = [self._order_manager.get_event_contract_dict()[i.name()] for i in self._contract_array]
+        for combo_name, amount in self._combo_order.items():
+            if combo_name in self._combo_def:
+                combo_def = self._combo_def[combo_name]
+                self._order_manager.register_combo_level_info(self._strategy_name,
+                                                              contract_array,
+                                                              combo_def,
+                                                              combo_name)
+                for i in range(len(combo_def)):
+                    self._order_manager.place_order(self._strategy_name,
+                                                    combo_name,
+                                                    i,
+                                                    contract_array[i],
+                                                    amount)
+            else:
+                # Some strategies do not have predefined combos;
+                # for them we simply place orders for individual contract.
+                raise ValueError('StrategyBase.send_order_live: Non-combo ordering not implemented yet.')
+    # % Record entered combo position locally
+    # obj.positionArr{1, comboId} = obj.positionArr{1, comboId} + comboUnit;
 
     def _calculate_new_average_entry_price(self,
                                            old_average_entry_price: float,
@@ -389,19 +455,6 @@ class StrategyBase(ABC):
         """
         self._mtm_history.append(new_mtm)
 
-    def get_combo_mtm_price(self, combo_name: str) -> float:
-        """
-        Retrieve combo mtm price.
-
-
-        Parameters
-        ----------
-        combo_name: str
-            Combo name.
-        """
-        combo_def = self._combo_def[combo_name]
-        return sum(i[0] * i[1] for i in zip(combo_def, self._mtm_price.values()))
-
     def has_pending_order(self) -> bool:
         """
         Check if there is any pending order.
@@ -461,7 +514,7 @@ class MACrossingStrategy(StrategyBase):
                  trade_combo: TradeCombos,
                  warmup_length: int,
                  initial_capital: float = 100.0,
-                 order_manager: brokerinterface.OrderManager = None) -> None:
+                 order_manager: brokerinterface.OrderManagerBase = None) -> None:
         """
         Initialize MA Crossing Strategy.
 
@@ -475,7 +528,7 @@ class MACrossingStrategy(StrategyBase):
             TradeCombos object that defines the combos to be traded. Note that if combo definition is missing,
             all trades from this strategy will be under a generic combo.
         warmup_length: int
-            Number of data points to 'burn'
+            Number of data points to 'burn'.
         initial_capital: float
             Initial capital.
         order_manager: OrderManager
@@ -483,7 +536,7 @@ class MACrossingStrategy(StrategyBase):
         """
         if len(trade_combo.get_combo_def()) != 1:
             raise ValueError('MACrossingStrategy.__init__: Too many combos specified.')
-        self._strategy_name = input_signal_array[0].name() + "_MACrossing"
+        self._strategy_name = input_signal_array[0].name() + self._strategy_signature
         self._long_signal_name, self._short_signal_name = self._deduce_signal_name(input_signal_array)
 
     def _deduce_signal_name(self, input_signal_array: List[Event]) -> (str, str):
@@ -528,6 +581,187 @@ class MACrossingStrategy(StrategyBase):
             return {combo_name: 0.0}
 
 
+class VAAStrategy(StrategyBase):
+    """
+    VAAStrategy
+    https://indexswingtrader.blogspot.com/2017/07/breadth-momentum-and-vigilant-asset.html
+    Given a top selection T and a breadth protection threshold B, for each month:
+    - Compute 13612W momentum for each asset
+    - Pick the best performing assets in the "risk-on" universe as top T
+    - Pick the best asset in the "risk-off" universe as safety asset for "cash"
+    - Compute the number of assets with non-positive momentum in the "risk-on" universe (b)
+    - Compute b/B and round down to multiples of 1/T as "cash fraction" CF for "easy trading"
+    - Replace CF of top T by "cash" asset as selected in step 3
+    """
+
+    _strategy_signature = 'VAA'
+
+    def __init__(self,
+                 signature_str: str,
+                 input_signal_array: List[Event],
+                 trade_combo: TradeCombos,
+                 warmup_length: int,
+                 initial_capital: float = 100.0,
+                 order_manager: brokerinterface.OrderManagerBase = None,
+                 **kwargs) -> None:
+        """
+        Initialize VAA rotation Strategy.
+
+        Parameters
+        ----------
+        signature_str: str
+            Unique signature of the strategy.
+        input_signal_array: List[Event]
+            List of event subscription that is required for trade decision.
+        trade_combo: TradeCombos
+            TradeCombos object that defines the combos to be traded. Note that if combo definition is missing,
+            all trades from this strategy will be under a generic combo.
+        warmup_length: int
+            Number of data points to 'burn'.
+        initial_capital: float
+            Initial capital.
+        order_manager: OrderManager
+            OrderManager object for live trading.
+
+        Other Parameters
+        ----------------
+        risk_on_size: int
+            Number of risk-on assets within the input signals.
+        num_assets_to_hold: int
+            Portfolio size in terms of number of holding.
+        breadth_protection_threshold: int
+            Threshold for protection.
+        weighting_scheme: WeightingScheme
+            Portfolio weighting scheme.
+        """
+        if len(trade_combo.get_contract_array()) < kwargs['num_assets_to_hold']:
+            raise ValueError('VAAStrategy.__init__: num_assets_to_hold must be smaller than universe size.')
+        if len(input_signal_array) != len(trade_combo.get_contract_array()):
+            raise ValueError('VAAStrategy.__init__: input_signal_array and trade combo contracts '
+                             'should be of the same size.')
+        all_input_names = [x.name() for x in input_signal_array]
+        self._strategy_name = ''.join(all_input_names) + "_" + self._strategy_signature
+        self._num_assets_to_hold = kwargs['num_assets_to_hold']
+        self._weighting_scheme = kwargs['weighting_scheme']
+        self._breadth_protection_threshold = kwargs['breadth_protection_threshold']
+        self._risk_on_names, self._risk_off_names = self.initialize_names(input_signal_array, kwargs['risk_on_size'])
+
+    def initialize_names(self,
+                         input_signal_array: List[Event],
+                         risk_on_size: int) -> (List[str], List[str]):
+        """
+        Initialize the risk-on and risk-off names.
+
+        Parameters
+        ----------
+        input_signal_array: List[Event]
+            List of event subscription that is required for trade decision.
+        risk_on_size: int
+            Number of risk-on assets within the input signals.
+
+        Returns
+        -------
+            List[str], List[str]
+                The risk-on and risk-off name lists, respectively.
+        """
+        output_on = []
+        output_off = []
+        for i in range(len(input_signal_array)):
+            output_on.append(input_signal_array[i].name()) if i < risk_on_size else \
+                output_off.append(input_signal_array[i].name())
+        return output_on, output_off
+
+    def get_risk_on_off_signals(self, is_on: bool) -> Dict[str, Deque[float]]:
+        """
+        Retrieve risk-on or risk-off signals.
+
+        Parameters
+        ----------
+        is_on: bool
+            Whether we are interested in risk-on or risk-off.
+
+        Returns
+        -------
+            Dict[str, Deque[float]]
+                A dictionary containing signals on the selection of names.
+        """
+        output = {}
+        names = self._risk_on_names if is_on else self._risk_off_names
+        for name in names:
+            output[name] = self.get_signal_by_name(name)
+        return output
+
+    def make_order_decision(self) -> Dict[str, float]:
+        # Identify top performing assets from the risk-on assets
+        on_signals = self.get_risk_on_off_signals(True)
+        top_risk_on_performers = sorted(
+            on_signals.items(), key=lambda e: e[1][-1], reverse=True)[:self._num_assets_to_hold]
+        # Identify top performing assets from the risk-off assets
+        off_signals = self.get_risk_on_off_signals(False)
+        top_risk_off_performers = sorted(
+            off_signals.items(), key=lambda e: e[1][-1], reverse=True)[0]
+        # Compute the number of assets with non-positive momentum in the "risk-on" universe
+        max_signals = map(max, on_signals.values())
+        num_non_positive = sum(i < 0.0 for i in max_signals)
+
+        rounded_cash_fraction = max(0.0, min(1.0, (1.0 / float(self._num_assets_to_hold))
+                                             * math.floor(float(num_non_positive) * float(self._num_assets_to_hold)
+                                                          / float(self._breadth_protection_threshold))))
+
+        # Rebalance - Buy new holdings
+        output = self.calculate_weight(top_risk_on_performers, top_risk_off_performers, rounded_cash_fraction)
+
+        # Rebalance - Sell old holdings
+        for pos_k, pos_v in self._contract_positions.items():
+            if (abs(pos_v) > 0.0) and (pos_k != self.CASH_ACCOUNT_NAME):
+                if pos_k not in output:
+                    output[pos_k] = -pos_v
+                else:
+                    output[pos_k] -= pos_v
+        return output
+
+    def calculate_weight(self,
+                         top_performers: List[str],
+                         top_risk_off_performer: str,
+                         rounded_cash_fraction: float) -> Dict[str, float]:
+        """
+        Calculate the weight to trade.
+
+        Parameters
+        ----------
+        top_performers: List[str]
+            Top risk-on performers' performances.
+        top_risk_off_performer: str
+            Top risk-off performer's performance.
+        rounded_cash_fraction: float
+            Rounded fraction to be held in cash.
+
+        Returns
+        -------
+            Dict[str, float]
+                Trade decision.
+        """
+        num_risk_on_to_buy = round(1.0 - rounded_cash_fraction) / (1.0 / float(self._num_assets_to_hold))
+        output = {}
+        num_risk_on_bought = 0
+        for t in top_performers:
+            if num_risk_on_bought < num_risk_on_to_buy:
+                contract_name = t[0].split('_')[0]
+                if self._weighting_scheme == WeightingScheme.Equal:
+                    asset_weight = 1.0 / self._num_assets_to_hold
+                else:
+                    raise ValueError('VAAStrategy.calculate_weight: unknown weighting scheme '
+                                     + str(self._weighting_scheme))
+                output[contract_name] = asset_weight
+                num_risk_on_bought += 1
+            else:
+                contract_name = top_risk_off_performer[0].split('_')[0]
+                output[contract_name] = rounded_cash_fraction\
+                    * self.get_mtm_history()[-1] / self.get_mtm_price_by_name(contract_name)
+                break
+        return output
+
+
 class SlotBase:
     """
     Base class for slot, to be derived for Strategy.
